@@ -8,18 +8,25 @@ use FindBin;
 use lib File::Spec->catdir( $FindBin::Bin, 'lib' );
 use Test::Smoke::Mailer;
 
-use vars qw( $VERSION );
-$VERSION = '0.001';
+use vars qw( $VERSION $conf );
+$VERSION = '0.004';
 
 my %opt = (
-    type => 'error',
-    ddir => cwd(),
-    to   => 'smokers-reports@perl.org',
-    v    => 0,
+    type    => undef,
+    ddir    => '',
+    to      => '', #'smokers-reports@perl.org',
+    cc      => '',
+    from    => '',
+    mserver => '',
+    v       => 0,
 
-    help => 0,
+    config  => '',
+    help    => 0,
+    man     => 0,
 );
 
+my $defaults = Test::Smoke::Mailer->config( 'all_defaults' );
+
 my %valid_type = map { $_ => 1 } qw( mail mailx sendmail Mail::Sendmail );
 
 =head1 NAME
@@ -30,12 +37,23 @@ mailrpt.pl - Send the smoke report by mail
 
     $ ./mailrpt.pl -t mailx -d ../perl-current [--help | more options]
 
+or
+
+    $ ./mailrpt.pl -c smokeperl_config
+
 =head1 OPTIONS
 
 Options depend on the B<type> option, exept for some.
 
 =over 4
 
+=item * B<Configuration file>
+
+    -c | --config <configfile> Use the settings from the configfile
+
+F<mailrpt.pl> can use the configuration file created by F<configsmoke.pl>.
+Other options can override the settings from the configuration file.
+
 =item * B<General options>
 
     -d | --ddir <directory>  Set the directory for the source-tree (cwd)
@@ -47,7 +65,7 @@ Options depend on the B<type> option, exept for some.
 
 =item * B<options for> -t mail/mailx
 
-none
+no extra options
 
 =item * B<options for> -t sendmail
 
@@ -68,16 +86,32 @@ GetOptions( \%opt,
     'from=s', 'mserver=s',
 
     'help|h', 'man|m',
+
+    'config|c=s',
 ) or do_pod2usage( verbose => 1 );
 
 $opt{man}  and do_pod2usage( verbose => 2, exitval => 0 );
 $opt{help} and do_pod2usage( verbose => 1, exitval => 0 );
 
+if ( $opt{config} && -f $opt{config} ) {
+    require $opt{config};
+
+    foreach my $option ( keys %opt ) {
+        if ( $option eq 'type' ) {
+            $opt{type} ||= $conf->{mail_type};
+        } elsif ( exists $conf->{ $option } ) {
+            $opt{ $option } ||= $conf->{ $option }
+        }
+    }
+}
+
+$opt{ $_ } ||= $defaults->{ $_ } foreach keys %$defaults;
+
 exists $valid_type{ $opt{type} } or do_pod2usage( verbose => 0 );
-$opt{ddir} or do_pod2usage( verbose => 0 );
+
+$opt{ddir} && -d $opt{ddir} or do_pod2usage( verbose => 0 );
 
 my $mailer = Test::Smoke::Mailer->new( $opt{type} => \%opt );
-
 $mailer->mail;
 
 sub do_pod2usage {
@@ -90,7 +124,8 @@ Use 'perldoc $0' for the documentation.
 Please install 'Pod::Usage' for easy access to the docs.
 
 EO_MSG
-        exit;
+        my %p2u_opt = @_;
+        exit( exists $p2u_opt{exitval} ? $p2u_opt{exitval} : 1 );
     } else {
         Pod::Usage::pod2usage( @_ );
     }
@@ -102,11 +137,25 @@ L<Test::Smoke::Mailer>, L<mkovz.pl>
 
 =head1 COPYRIGHT
 
-(c) 2002, All rights reserved.
+(c) 2002-2003, All rights reserved.
 
   * Abe Timmerman <abeltje@cpan.org>
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
 
+See:
+
+=over 4
+
+=item * http://www.perl.com/perl/misc/Artistic.html
+
+=item * http://www.gnu.org/copyleft/gpl.html
+
+=back
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+
 =cut
